@@ -21,6 +21,23 @@ func BuildCaseInsensitiveLike(key, value string) builder.Cond {
 	return builder.Like{"UPPER(" + key + ")", strings.ToUpper(value)}
 }
 
+// BuildCaseInsensitiveIn returns a condition to check if the given value is in the given values case-insensitively.
+// Handles especially SQLite correctly as UPPER there only transforms ASCII letters.
+func BuildCaseInsensitiveIn(key string, values []string) builder.Cond {
+	uppers := make([]string, 0, len(values))
+	if setting.Database.Type.IsSQLite3() {
+		for _, value := range values {
+			uppers = append(uppers, util.ToUpperASCII(value))
+		}
+	} else {
+		for _, value := range values {
+			uppers = append(uppers, strings.ToUpper(value))
+		}
+	}
+
+	return builder.In("UPPER("+key+")", uppers)
+}
+
 // BuilderDialect returns the xorm.Builder dialect of the engine
 func BuilderDialect() string {
 	switch {
