@@ -4,6 +4,8 @@
 package issues
 
 import (
+	"context"
+
 	"code.gitea.io/gitea/models/db"
 	user_model "code.gitea.io/gitea/models/user"
 )
@@ -17,16 +19,16 @@ type IssueLockOptions struct {
 
 // LockIssue locks an issue. This would limit commenting abilities to
 // users with write access to the repo
-func LockIssue(opts *IssueLockOptions) error {
-	return updateIssueLock(opts, true)
+func LockIssue(ctx context.Context, opts *IssueLockOptions) error {
+	return updateIssueLock(ctx, opts, true)
 }
 
 // UnlockIssue unlocks a previously locked issue.
-func UnlockIssue(opts *IssueLockOptions) error {
-	return updateIssueLock(opts, false)
+func UnlockIssue(ctx context.Context, opts *IssueLockOptions) error {
+	return updateIssueLock(ctx, opts, false)
 }
 
-func updateIssueLock(opts *IssueLockOptions, lock bool) error {
+func updateIssueLock(ctx context.Context, opts *IssueLockOptions, lock bool) error {
 	if opts.Issue.IsLocked == lock {
 		return nil
 	}
@@ -39,7 +41,7 @@ func updateIssueLock(opts *IssueLockOptions, lock bool) error {
 		commentType = CommentTypeUnlock
 	}
 
-	ctx, committer, err := db.TxContext(db.DefaultContext)
+	ctx, committer, err := db.TxContext(ctx)
 	if err != nil {
 		return err
 	}
