@@ -4,24 +4,27 @@
 package conan
 
 import (
+	std_ctx "context"
 	"fmt"
 	"io"
 	"net/http"
 	"strings"
 	"time"
 
+	auth_model "code.gitea.io/gitea/models/auth"
 	"code.gitea.io/gitea/models/db"
 	packages_model "code.gitea.io/gitea/models/packages"
 	conan_model "code.gitea.io/gitea/models/packages/conan"
 	"code.gitea.io/gitea/modules/container"
-	"code.gitea.io/gitea/modules/context"
 	"code.gitea.io/gitea/modules/json"
 	"code.gitea.io/gitea/modules/log"
-	"code.gitea.io/gitea/modules/notification"
 	packages_module "code.gitea.io/gitea/modules/packages"
 	conan_module "code.gitea.io/gitea/modules/packages/conan"
 	"code.gitea.io/gitea/modules/setting"
 	"code.gitea.io/gitea/routers/api/packages/helper"
+	auth_service "code.gitea.io/gitea/services/auth"
+	"code.gitea.io/gitea/services/context"
+	notify_service "code.gitea.io/gitea/services/notify"
 	packages_service "code.gitea.io/gitea/services/packages"
 )
 
@@ -71,11 +74,11 @@ func baseURL(ctx *context.Context) string {
 // ExtractPathParameters is a middleware to extract common parameters from path
 func ExtractPathParameters(ctx *context.Context) {
 	rref, err := conan_module.NewRecipeReference(
-		ctx.Params("name"),
-		ctx.Params("version"),
-		ctx.Params("user"),
-		ctx.Params("channel"),
-		ctx.Params("recipe_revision"),
+		ctx.PathParam("name"),
+		ctx.PathParam("version"),
+		ctx.PathParam("user"),
+		ctx.PathParam("channel"),
+		ctx.PathParam("recipe_revision"),
 	)
 	if err != nil {
 		apiError(ctx, http.StatusBadRequest, err)
@@ -84,14 +87,14 @@ func ExtractPathParameters(ctx *context.Context) {
 
 	ctx.Data[recipeReferenceKey] = rref
 
-	reference := ctx.Params("package_reference")
+	reference := ctx.PathParam("package_reference")
 
 	var pref *conan_module.PackageReference
 	if reference != "" {
 		pref, err = conan_module.NewPackageReference(
 			rref,
 			reference,
-			ctx.Params("package_revision"),
+			ctx.PathParam("package_revision"),
 		)
 		if err != nil {
 			apiError(ctx, http.StatusBadRequest, err)
@@ -116,7 +119,20 @@ func Authenticate(ctx *context.Context) {
 		return
 	}
 
-	token, err := packages_service.CreateAuthorizationToken(ctx.Doer)
+	packageScope := auth_service.GetAccessScope(ctx.Data)
+	if has, err := packageScope.HasAnyScope(
+		auth_model.AccessTokenScopeReadPackage,
+		auth_model.AccessTokenScopeWritePackage,
+		auth_model.AccessTokenScopeAll,
+	); !has {
+		if err != nil {
+			log.Error("Error checking access scope: %v", err)
+		}
+		apiError(ctx, http.StatusForbidden, nil)
+		return
+	}
+
+	token, err := packages_service.CreateAuthorizationToken(ctx.Doer, packageScope)
 	if err != nil {
 		apiError(ctx, http.StatusInternalServerError, err)
 		return
@@ -129,9 +145,23 @@ func Authenticate(ctx *context.Context) {
 func CheckCredentials(ctx *context.Context) {
 	if ctx.Doer == nil {
 		ctx.Status(http.StatusUnauthorized)
-	} else {
-		ctx.Status(http.StatusOK)
+		return
 	}
+
+	packageScope := auth_service.GetAccessScope(ctx.Data)
+	if has, err := packageScope.HasAnyScope(
+		auth_model.AccessTokenScopeReadPackage,
+		auth_model.AccessTokenScopeWritePackage,
+		auth_model.AccessTokenScopeAll,
+	); !has {
+		if err != nil {
+			log.Error("Error checking access scope: %v", err)
+		}
+		ctx.Status(http.StatusForbidden)
+		return
+	}
+
+	ctx.Status(http.StatusOK)
 }
 
 // RecipeSnapshot displays the recipe files with their md5 hash
@@ -303,18 +333,18 @@ func uploadFile(ctx *context.Context, fileFilter container.Set[string], fileKey
 	rref := ctx.Data[recipeReferenceKey].(*conan_module.RecipeReference)
 	pref := ctx.Data[packageReferenceKey].(*conan_module.PackageReference)
 
-	filename := ctx.Params("filename")
+	filename := ctx.PathParam("filename")
 	if !fileFilter.Contains(filename) {
 		apiError(ctx, http.StatusBadRequest, nil)
 		return
 	}
 
-	upload, close, err := ctx.UploadStream()
+	upload, needToClose, err := ctx.UploadStream()
 	if err != nil {
 		apiError(ctx, http.StatusBadRequest, err)
 		return
 	}
-	if close {
+	if needToClose {
 		defer upload.Close()
 	}
 
@@ -325,13 +355,8 @@ func uploadFile(ctx *context.Context, fileFilter container.Set[string], fileKey
 	}
 	defer buf.Close()
 
-	if buf.Size() == 0 {
-		// ignore empty uploads, second request contains content
-		jsonResponse(ctx, http.StatusOK, nil)
-		return
-	}
-
 	isConanfileFile := filename == conanfileFile
+	isConaninfoFile := filename == conaninfoFile
 
 	pci := &packages_service.PackageCreationInfo{
 		PackageInfo: packages_service.PackageInfo{
@@ -363,7 +388,7 @@ func uploadFile(ctx *context.Context, fileFilter container.Set[string], fileKey
 		pfci.Properties[conan_module.PropertyPackageRevision] = pref.RevisionOrDefault()
 	}
 
-	if isConanfileFile || filename == conaninfoFile {
+	if isConanfileFile || isConaninfoFile {
 		if isConanfileFile {
 			metadata, err := conan_module.ParseConanfile(buf)
 			if err != nil {
@@ -412,13 +437,14 @@ func uploadFile(ctx *context.Context, fileFilter container.Set[string], fileKey
 	}
 
 	_, _, err = packages_service.CreatePackageOrAddFileToExisting(
+		ctx,
 		pci,
 		pfci,
 	)
 	if err != nil {
 		switch err {
 		case packages_model.ErrDuplicatePackageFile:
-			apiError(ctx, http.StatusBadRequest, err)
+			apiError(ctx, http.StatusConflict, err)
 		case packages_service.ErrQuotaTotalCount, packages_service.ErrQuotaTypeSize, packages_service.ErrQuotaTotalSize:
 			apiError(ctx, http.StatusForbidden, err)
 		default:
@@ -447,13 +473,13 @@ func DownloadPackageFile(ctx *context.Context) {
 func downloadFile(ctx *context.Context, fileFilter container.Set[string], fileKey string) {
 	rref := ctx.Data[recipeReferenceKey].(*conan_module.RecipeReference)
 
-	filename := ctx.Params("filename")
+	filename := ctx.PathParam("filename")
 	if !fileFilter.Contains(filename) {
 		apiError(ctx, http.StatusBadRequest, nil)
 		return
 	}
 
-	s, pf, err := packages_service.GetFileStreamByPackageNameAndVersion(
+	s, u, pf, err := packages_service.GetFileStreamByPackageNameAndVersion(
 		ctx,
 		&packages_service.PackageInfo{
 			Owner:       ctx.Package.Owner,
@@ -474,12 +500,8 @@ func downloadFile(ctx *context.Context, fileFilter container.Set[string], fileKe
 		apiError(ctx, http.StatusInternalServerError, err)
 		return
 	}
-	defer s.Close()
 
-	ctx.ServeContent(s, &context.ServeHeaderOptions{
-		Filename:     pf.Name,
-		LastModified: pf.CreatedUnix.AsLocalTime(),
-	})
+	helper.ServePackageFile(ctx, s, u, pf)
 }
 
 // DeleteRecipeV1 deletes the requested recipe(s)
@@ -606,72 +628,67 @@ func DeletePackageV2(ctx *context.Context) {
 }
 
 func deleteRecipeOrPackage(apictx *context.Context, rref *conan_module.RecipeReference, ignoreRecipeRevision bool, pref *conan_module.PackageReference, ignorePackageRevision bool) error {
-	ctx, committer, err := db.TxContext(db.DefaultContext)
-	if err != nil {
-		return err
-	}
-	defer committer.Close()
+	var pd *packages_model.PackageDescriptor
+	versionDeleted := false
 
-	pv, err := packages_model.GetVersionByNameAndVersion(ctx, apictx.Package.Owner.ID, packages_model.TypeConan, rref.Name, rref.Version)
-	if err != nil {
-		return err
-	}
-
-	pd, err := packages_model.GetPackageDescriptor(ctx, pv)
-	if err != nil {
-		return err
-	}
-
-	filter := map[string]string{
-		conan_module.PropertyRecipeUser:    rref.User,
-		conan_module.PropertyRecipeChannel: rref.Channel,
-	}
-	if !ignoreRecipeRevision {
-		filter[conan_module.PropertyRecipeRevision] = rref.RevisionOrDefault()
-	}
-	if pref != nil {
-		filter[conan_module.PropertyPackageReference] = pref.Reference
-		if !ignorePackageRevision {
-			filter[conan_module.PropertyPackageRevision] = pref.RevisionOrDefault()
+	err := db.WithTx(apictx, func(ctx std_ctx.Context) error {
+		pv, err := packages_model.GetVersionByNameAndVersion(ctx, apictx.Package.Owner.ID, packages_model.TypeConan, rref.Name, rref.Version)
+		if err != nil {
+			return err
 		}
-	}
 
-	pfs, _, err := packages_model.SearchFiles(ctx, &packages_model.PackageFileSearchOptions{
-		VersionID:  pv.ID,
-		Properties: filter,
+		pd, err = packages_model.GetPackageDescriptor(ctx, pv)
+		if err != nil {
+			return err
+		}
+
+		filter := map[string]string{
+			conan_module.PropertyRecipeUser:    rref.User,
+			conan_module.PropertyRecipeChannel: rref.Channel,
+		}
+		if !ignoreRecipeRevision {
+			filter[conan_module.PropertyRecipeRevision] = rref.RevisionOrDefault()
+		}
+		if pref != nil {
+			filter[conan_module.PropertyPackageReference] = pref.Reference
+			if !ignorePackageRevision {
+				filter[conan_module.PropertyPackageRevision] = pref.RevisionOrDefault()
+			}
+		}
+
+		pfs, _, err := packages_model.SearchFiles(ctx, &packages_model.PackageFileSearchOptions{
+			VersionID:  pv.ID,
+			Properties: filter,
+		})
+		if err != nil {
+			return err
+		}
+		if len(pfs) == 0 {
+			return conan_model.ErrPackageReferenceNotExist
+		}
+
+		for _, pf := range pfs {
+			if err := packages_service.DeletePackageFile(ctx, pf); err != nil {
+				return err
+			}
+		}
+		has, err := packages_model.HasVersionFileReferences(ctx, pv.ID)
+		if err != nil {
+			return err
+		}
+		if !has {
+			versionDeleted = true
+
+			return packages_service.DeletePackageVersionAndReferences(ctx, pv)
+		}
+		return nil
 	})
 	if err != nil {
 		return err
 	}
-	if len(pfs) == 0 {
-		return conan_model.ErrPackageReferenceNotExist
-	}
-
-	for _, pf := range pfs {
-		if err := packages_service.DeletePackageFile(ctx, pf); err != nil {
-			return err
-		}
-	}
-
-	versionDeleted := false
-	has, err := packages_model.HasVersionFileReferences(ctx, pv.ID)
-	if err != nil {
-		return err
-	}
-	if !has {
-		versionDeleted = true
-
-		if err := packages_service.DeletePackageVersionAndReferences(ctx, pv); err != nil {
-			return err
-		}
-	}
-
-	if err := committer.Commit(); err != nil {
-		return err
-	}
 
 	if versionDeleted {
-		notification.NotifyPackageDelete(apictx, apictx.Doer, pd)
+		notify_service.PackageDelete(apictx, apictx.Doer, pd)
 	}
 
 	return nil
